##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################


import LoggerBase
from procps.linuxproc import linuxproc
import resource
import os, time

def meminfo(mem=None):
    if not mem: mem = linuxproc.meminfo()
    swpd  = mem['swaptotal'] - mem['swapfree']
    return mem['swapfree'], swpd, mem['memfree'], mem['buffers'], mem['cached']

def getcpu(stat=None):
    if not stat: stat = linuxproc.stat()
    cpu = stat['cpu']
    return cpu['user'], cpu['nice'], cpu['system'], cpu['idle']

def getswap(stat=None):
    if not stat: stat = linuxproc.stat()
    swap = stat['swap']
    return swap['in'], swap['out']

def getpage(stat=None):
    if not stat: stat = linuxproc.stat()
    page = stat['page']
    return page['in'], page['out']

def getall():
    stat = linuxproc.stat()
    d = {}
    lr, lb, ls = map(lambda x: len(x), linuxproc.getrunners())
    swapfree, mswap, mfree, mbuf, mcache = meminfo()
    swapin, swapout = getswap(stat)
    pgin, pgout = getpage(stat)
    intr = stat['intr']['interrupts']
    ctxt = stat['ctxt']['context_switches']
    cuse, cnice, csys, cidl = getcpu(stat)
    loc = locals()
    for name in ('lr', 'lb', 'ls', 'mswap', 'mfree', 'mbuf', 'mcache',
                 'swapin', 'swapout', 'pgin', 'pgout', 'intr', 'ctxt', 'cuse',
                 'cnice', 'csys', 'cidl', 'swapfree'):
        d[name] = loc[name]
    return d

tog = 0
stats = [getall(), None]

class LinuxLoggerBase(LoggerBase.LoggerBase):
    """ LinuxLogger base class """

    platforms = ('linux2', 'linux-i386')

    def doLog(self,*args,**kw):
        # system stats
        delay = kw['delay']
        global tog
        global stats
        KB_PER_PAGE = 1
        tog = not tog
        stat = stats[tog] = getall()
        old =  stats[not tog]
        
        duse = (stat['cuse']-old['cuse']) + (stat['cnice']-old['cnice'])
        dsys = (stat['csys']-old['csys'])
        didl = (stat['cidl']-old['cidl'])

        if didl < 0:
            didl = 0

        Div = duse + dsys + didl
        divo2 = Div/2
        pero2 = delay/2
        fcusr = (100L * duse + divo2)/Div
        fcsys = (100L * dsys + divo2)/Div
        fcidle = (100L * didl + divo2)/Div
        fswapin  = ((stat['swapin']-old['swapin'])   *KB_PER_PAGE+pero2)/delay
        fswapout = ((stat['swapout']-old['swapout']) *KB_PER_PAGE+pero2)/delay
        fpgin    = ((stat['pgin']-old['pgin'])                   +pero2)/delay
        fpgout   = ((stat['pgout']-old['pgout'])                 +pero2)/delay
        mfree, swapfree = stat['mfree'], stat['swapfree']
        s = "%d %d %d %d %d %d %d %d %d" % (
                fcusr, fcsys, fcidle, # user, system, idle
                fswapin,fswapout, # swapin, swapout  
                fpgin,fpgout, # pagein, pageout
                mfree, mfree+swapfree # freereal, freevirt
            )
        self.log('S', s)

        # process stats
        pagesize = resource.getpagesize()
        statm = linuxproc.self_statm()
        psize = statm.get('size') * pagesize/1024
        refcounts = 0 # fixme
        s = "%d %d" % (psize, refcounts)
        self.log('P', s)

#######################################################################
#
#######################################################################

class ExternalLinuxLogger(LinuxLoggerBase):
    """ external logger """

    logname = os.path.join(CLIENT_HOME, "forensic.log")    

    def log(self, stage, s):
        fp = open(self.logname, 'a')
        fp.write("%s 000000000 %s" % (stage, self.format_log(s)))
        fp.close()

#######################################################################
#
#######################################################################

from ZServer import DebugLogger 

class BigMLinuxLogger(LinuxLoggerBase):
    """ logger for Zope Big-M logger """

    def log(self, stage, s):
        DebugLogger.log(stage,'000000000', s)


