#!/bin/python
##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
""" python vmstat for linux, requires linuxproc.py module """
__version__ = "1.0"
import linuxproc
import sys, time
HEIGHT = 24
DIGITS = {}
for d  in "1234567890":
    DIGITS[d] = 1

def showheader(timestamp):
    head="%8s%28s%8s%12s%11s%12s" % ('procs','memory','swap','io','system',
                                     'cpu')
    thead = head +"%12s" % 'time'
    print timestamp and thead or head
    s = "%2s %2s %2s %6s %6s %6s %6s %3s %3s %5s %5s %4s %5s %3s %3s %3s"
    items = (
        'r','b','w','swpd','free','buff','cache','si','so','bi','bo','in',
        'cs','us','sy','id'
        )
    print s % items

def meminfo(mem=None):
    if not mem: mem = linuxproc.meminfo()
    swpd  = mem['swaptotal'] - mem['swapfree']
    return swpd, mem['memfree'], mem['buffers'], mem['cached']

def getcpu(stat=None):
    if not stat: stat = linuxproc.stat()
    cpu = stat['cpu']
    return cpu['user'], cpu['nice'], cpu['system'], cpu['idle']

def getswap(stat=None):
    if not stat: stat = linuxproc.stat()
    swap = stat['swap']
    return swap['in'], swap['out']

def getpage(stat=None):
    if not stat: stat = linuxproc.stat()
    page = stat['page']
    return page['in'], page['out']

def getall():
    stat = linuxproc.stat()
    d = {}
    lr, lb, ls = map(lambda x: len(x), linuxproc.getrunners())
    mswap, mfree, mbuf, mcache = meminfo()
    swapin, swapout = getswap(stat)
    pgin, pgout = getpage(stat)
    intr = stat['intr']['interrupts']
    ctxt = stat['ctxt']['context_switches']
    cuse, cnice, csys, cidl = getcpu(stat)
    loc = locals()
    for name in ('lr', 'lb', 'ls', 'mswap', 'mfree', 'mbuf', 'mcache',
                 'swapin', 'swapout', 'pgin', 'pgout', 'intr', 'ctxt', 'cuse',
                 'cnice', 'csys', 'cidl'):
        d[name] = loc[name]
    return d

def main(headers, count, delay, timestamp):
    i = 0
    HZ = 100L
    KB_PER_PAGE = 1
    showheader(timestamp)
    l = [None, None]
    stat = l[0] = getall()
    duse = stat['cuse'] + stat['cnice']
    dsys = stat['csys']
    didl = stat['cidl']
    Div = duse + dsys + didl
    divo2 = Div/2
    fswapin  = (stat['swapin']  * KB_PER_PAGE * HZ + divo2)/Div
    fswapout = (stat['swapout'] * KB_PER_PAGE * HZ + divo2)/Div
    fpgin    = (stat['pgin']                  * HZ + divo2)/Div
    fpgout   = (stat['pgout']                 * HZ + divo2)/Div
    fintr    = (stat['intr']                  * HZ + divo2)/Div
    fctxt    = (stat['ctxt']                  * HZ + divo2)/Div
    fcusr    = (100L * duse + divo2)/Div
    fcsys    = (100L * dsys + divo2)/Div
    fcidle   = (100L * didl + divo2)/Div
    flr      = stat['lr'] - 1 # minus ourselves
    flb      = stat['lb']
    fls      = stat['ls']
    fmswap   = stat['mswap']
    fmfree   = stat['mfree']
    fmbuf    = stat['mbuf']
    fmcache  = stat['mcache']
    output(
        flr, flb, fls, fmswap, fmfree, fmbuf, fmcache, fswapin, fswapout,
        fpgin, fpgout, fintr, fctxt, fcusr, fcsys, fcidle,
        timestamp and time.asctime() or ''
        )
    
    # if we're repeating, do it below
    tog = 0
    i = 0
    while 1:
        i = i + 1
        if not delay: break
        if count and i == count: break
        tog = not tog
        time.sleep(delay)
        if i % HEIGHT == 0 and headers:
            showheader(timestamp)
        stat = l[tog] = getall()
        old  = l[not tog]
        duse = (stat['cuse']-old['cuse']) + (stat['cnice']-old['cnice'])
        dsys = (stat['csys']-old['csys'])
        didl = (stat['cidl']-old['cidl'])
        # idle may run backwards (kernel "feature")
        if didl < 0:
            didl = 0
        Div = duse + dsys + didl
        divo2 = Div/2
        pero2 = delay/2
        flr      = stat['lr'] - 1 # minus ourselves
        flb      = stat['lb']
        fls      = stat['ls']
        fmswap   = stat['mswap']
        fmfree   = stat['mfree']
        fmbuf    = stat['mbuf']
        fmcache  = stat['mcache']
        fswapin  = ((stat['swapin']-old['swapin'])   *KB_PER_PAGE+pero2)/delay
        fswapout = ((stat['swapout']-old['swapout']) *KB_PER_PAGE+pero2)/delay
        fpgin    = ((stat['pgin']-old['pgin'])                   +pero2)/delay
        fpgout   = ((stat['pgout']-old['pgout'])                 +pero2)/delay
        fintr    = ((stat['intr']-old['intr'])                   +pero2)/delay
        fctxt    = ((stat['ctxt']-old['ctxt'])                   +pero2)/delay
        fcusr    = (100L * duse + divo2)/Div
        fcsys    = (100L * dsys + divo2)/Div
        fcidle   = (100L * didl + divo2)/Div
        output(
            flr, flb, fls, fmswap, fmfree, fmbuf, fmcache, fswapin, fswapout,
            fpgin, fpgout, fintr, fctxt, fcusr, fcsys, fcidle,
            timestamp and time.asctime() or ''
            )
def output(*args):
    tfmt="%2u %2u %2u %6u %6u %6u %6u %3u %3u %5u %5u %4u %5u %3u %3u %3u %s"
    fmt = tfmt[:-3]
    if len(args) == 17:
        print tfmt % args
    elif len(args) == 16:
        print fmt % args

def write_tstamp():
    print time.asctime()

def usage():
    print "usage: %s [-V] [-n] [-t] [delay [count]]" % sys.argv[0]
    print "            -V prints version."
    print "            -n causes the headers not to be reprinted regularly."
    print "            -t causes a timestamp to be printed."
    print "            delay is the delay between updates in seconds."
    print "            count is the number of updates."
    
if __name__ == '__main__':
    delay_seen = None
    headers    = 1
    timestamps = 0
    count      = 0
    delay      = 0
    for arg in sys.argv[1:]:
        if arg[0] != '-':
            for char in arg:
                if not DIGITS.has_key(char):
                    usage()
                    sys.exit(1)
            if delay_seen:
                count = int(arg)
            else:
                delay = int(arg)
                delay_seen = 1
        elif arg[1:] and arg[1] == 'V':
            print sys.argv[0], 'version %s' % __version__
            sys.exit(0)
        elif arg[1:] and arg[1] == 'n':
            headers = 0
        elif arg[1:] and arg[1] == 't':
            timestamps = 1
        else:
            usage()
            sys.exit(1)
    main(headers, count, delay, timestamps)

